/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "log.h"

#include <cstdlib>
#include <cstring>

#include <dlfcn.h>

static int injectLibrary(const char path[])
{
	typedef int inject_func();

	int res = -1;
	void *lib = nullptr;
	inject_func *inject = nullptr;
	const char *inject_sym = "dotnet_launcher_inject";

	_INFO("Inject %s library", path);

	// Current implementation of heaptrack CLR profiler requires RTLD_GLOBAL
	lib = dlopen(path, RTLD_NOW | RTLD_GLOBAL);
	if (lib == nullptr) {
		_ERR("%s", dlerror());
		return res;
	}

	inject = reinterpret_cast<inject_func*>(dlsym(lib, inject_sym));
	if (inject == nullptr) {
		_ERR("%s is not found in the %s", inject_sym, path);
		return res;
	}

	res = inject();
	return res;
}

int checkInjection()
{
	int res = -1;
	char *env = nullptr;
	char* injectableLibs = nullptr;
	const char *delim = ", ";
	char *lib = nullptr;
	char *saveptr = nullptr;

	env = getenv("DOTNET_LAUNCHER_INJECT");
	if (env == nullptr) {
		res = 0;
		return res;
	}

	_INFO("##### Perform injection #########");

	injectableLibs = strdup(env);
	if (injectableLibs == nullptr) {
		_ERR("Fail to allocate memory for injectable library paths\n");
		return res;
	}

	res = 0;
	lib = strtok_r(injectableLibs, delim, &saveptr);
	for(; lib != nullptr; lib = strtok_r(nullptr, delim, &saveptr)) {
		if (injectLibrary(lib) != 0) {
			res = -1;
			break;
		}
	}

	if (res == 0) {
		_INFO("##### Injection finished #########");
	} else {
		_INFO("##### Injection failed #########");
	}

	free(injectableLibs);
	return res;
}
